/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include "RatingList.h"

using namespace xmltio;
using namespace airobots;
using namespace std;

RatingList::RatingList() {
	values = map<RatingList::XcfHashKey, double>();
	updateNeeded = false;
}

RatingList::~RatingList() {
}

void RatingList::sanitizeLocation(xmltio::Location & loc) const
{
	XPath xp("RATING");
	while (XPathIterator i = xp.evaluate(loc)) {
		Location l(*i);
		loc.remove(l);
	}
}



std::string RatingList::getTemplate()
{
	return "<" + getBaseTag() + ">"
			+ "</" + getBaseTag() + ">";
}



void RatingList::fillInto(xmltio::Location & loc) const
{
	Type::fillInto(loc);

	map<RatingList::XcfHashKey, double>::const_iterator i = values.begin();
	for (; i != values.end(); ++i) {
		RatingList::XcfHashKey key = i->first;
		double value = i->second;

		Location rating("<RATING/>", "RATING");
		rating[XPath("@anchorId1")] = key.key1;
		rating[XPath("@anchorId2")] = key.key2;
		rating[XPath("@value")] = value;

		loc.add(rating);
	}
}



void RatingList::fromLocation(const xmltio::Location & loc, RatingList & list)
{
	Type::fromLocation(loc, list);

	for (XPathIterator i = XPath("RATING").evaluate(loc); i; ++i) {
		Location l(*i);

		string key1 = extract<string>(l[XPath("@anchorId1")]);
		string key2 = extract<string>(l[XPath("@anchorId2")]);
		double value = extract<double>(l[XPath("@value")]);

		list.addRating(key1, key2, value);
	}
}



std::string RatingList::getBaseTag()
{
	return "RATINGS";
}



void RatingList::clearValues()
{
	values.clear();
}



void RatingList::setTolerance(double tol)
{
	tolerance = tol;
}



void RatingList::removeID(std::string id)
{
	map<RatingList::XcfHashKey, double>::iterator i = values.begin();
	for (; i != values.end(); ++i) {
		if (i->first.key1.compare(id) == 0) {
			values.erase(i);
			continue;
		}
		if (i->first.key2.compare(id) == 0) {
			values.erase(i);
			continue;
		}
	}
}



int RatingList::size()
{
	return values.size();
}



double RatingList::getRating(std::string key1, std::string key2)
{
	RatingList::XcfHashKey key;
	key.key1 = key1;
	key.key2 = key2;
	map<RatingList::XcfHashKey, double>::const_iterator it;
	if ((it = values.find(key)) != values.end()) {
		return it->second;
	}
	return -1;
}



void RatingList::setUpdateNeeded(bool needed)
{
	updateNeeded = needed;
}



bool RatingList::shouldBeUpdated()
{
	return updateNeeded;
}



void RatingList::addRating(RatingList::XcfHashKey key, double value)
{
	map<RatingList::XcfHashKey, double>::const_iterator it;
	if ((it = values.find(key)) != values.end()) {
		double oldValue = it->second;
		if (fabs(oldValue - value) > tolerance) {
			values[key] = value;
			updateNeeded = true;
		}
	} else {
		values[key] = value;
		updateNeeded = true;
	}
}



void RatingList::addRating(std::string key1, std::string key2, double value)
{
	RatingList::XcfHashKey key;
	key.key1 = key1;
	key.key2 = key2;
	addRating(key, value);
}



void RatingList::addRatingWOC(RatingList::XcfHashKey key, double value)
{
	values[key] = value;
}




std::map<RatingList::XcfHashKey,double> RatingList::getRatings()
{
	return values;
}


