/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.object;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Vector;

import de.unibi.airobots.btl.Type;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;

/**
 * Results of the object recognition. This class is meat so define the location
 * of the object in the detector's camera image! The given polygon describes the
 * objects's location in pixel coordinates! If you want to define an object in
 * world coordinates use {@link ObjectPositionData}.
 * 
 * @author lziegler
 */
public class ObjectLocationList extends Type implements
		List<ObjectLocationData> {

	/**
	 * 
	 */
	private static final long serialVersionUID = 5404915104164596108L;

	List<ObjectLocationData> list = new Vector<ObjectLocationData>();

	/**
	 * Default constructor.
	 */
	public ObjectLocationList() {
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "OBJECTLOCATIONLIST";
	}

	/**
	 * Fills an {@link ObjectLocationList} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to fill an object from.
	 * @param type
	 *            The {@link ObjectLocationList} object to fill with all the
	 *            information given by the {@link Element} object.
	 */
	public static void fromElement(Element objectElement,
			ObjectLocationList type) {

		try {
			// check hypotheses
			Nodes nodes = objectElement.query(ObjectLocationData.getBaseTag());
			for (int i = 0; i < nodes.size(); i++) {
				Node node = nodes.get(i);
				if (node instanceof Element) {

					ObjectLocationData data = new ObjectLocationData();
					ObjectLocationData.fromElement((Element) node, data);
					type.add(data);
				}
			}

		} catch (NullPointerException ex) {
			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {

		super.fillInto(parent);

		for (ObjectLocationData data : this) {
			Element dataElem = new Element(ObjectLocationData.getBaseTag());
			data.fillInto(dataElem);
			parent.appendChild(dataElem);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String text = "[OBJECTLOCATIONLIST\n";
		for (ObjectLocationData data : list) {
			text += "\t" + data.toString() + "\n";
		}
		text += "]";
		return text;
	}

	@Override
	public boolean add(ObjectLocationData e) {
		return list.add(e);
	}

	@Override
	public void add(int index, ObjectLocationData element) {
		list.add(index, element);
	}

	@Override
	public boolean addAll(Collection<? extends ObjectLocationData> c) {
		return list.addAll(c);
	}

	@Override
	public boolean addAll(int index, Collection<? extends ObjectLocationData> c) {
		return list.addAll(index, c);
	}

	@Override
	public void clear() {
		list.clear();
	}

	@Override
	public boolean contains(Object o) {
		return list.contains(o);
	}

	@Override
	public boolean containsAll(Collection<?> c) {
		return list.containsAll(c);
	}

	@Override
	public ObjectLocationData get(int index) {
		return list.get(index);
	}

	@Override
	public int indexOf(Object o) {
		return list.indexOf(o);
	}

	@Override
	public boolean isEmpty() {
		return list.isEmpty();
	}

	@Override
	public Iterator<ObjectLocationData> iterator() {
		return list.iterator();
	}

	@Override
	public int lastIndexOf(Object o) {
		return list.lastIndexOf(o);
	}

	@Override
	public ListIterator<ObjectLocationData> listIterator() {
		return list.listIterator();
	}

	@Override
	public ListIterator<ObjectLocationData> listIterator(int index) {
		return list.listIterator(index);
	}

	@Override
	public boolean remove(Object o) {
		return list.remove(o);
	}

	@Override
	public ObjectLocationData remove(int index) {
		return list.remove(index);
	}

	@Override
	public boolean removeAll(Collection<?> c) {
		return list.removeAll(c);
	}

	@Override
	public boolean retainAll(Collection<?> c) {
		return list.retainAll(c);
	}

	@Override
	public ObjectLocationData set(int index, ObjectLocationData element) {
		return list.set(index, element);
	}

	@Override
	public int size() {
		return list.size();
	}

	@Override
	public List<ObjectLocationData> subList(int fromIndex, int toIndex) {
		return list.subList(fromIndex, toIndex);
	}

	@Override
	public Object[] toArray() {
		return list.toArray();
	}

	@Override
	public <T> T[] toArray(T[] a) {
		return list.toArray(a);
	}
}
