/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/data/person/Legs.h"
#include "../../utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(LegsTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		Legs original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setAngle(21312.78,DEGREE);
		original.setDistance(-1.023,METER);
		original.setAngleVariance(0.45,DEGREE);
		original.setDistanceVariance(1.34,METER);
		original.setNumInvolved(2134);
		original.setInvolved1(4543);
		original.setInvolved2(1223);
		original.setPair(true);

		Location originalLoc(Legs::getTemplate(), Legs::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true);

		Legs parsed = extract<Legs>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_CLOSE(parsed.getAngle(RADIAN), original.getAngle(RADIAN), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getDistance(MILLIMETER), original.getDistance(MILLIMETER), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getAngleVariance(RADIAN), original.getAngleVariance(RADIAN), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getDistanceVariance(MILLIMETER), original.getDistanceVariance(MILLIMETER), tolerance);
		BOOST_REQUIRE_EQUAL(parsed.getNumInvovled(), original.getNumInvovled());
		BOOST_REQUIRE_EQUAL(parsed.getInvolved1(), original.getInvolved1());
		BOOST_REQUIRE_EQUAL(parsed.getInvolved2(), original.getInvolved2());
		BOOST_REQUIRE_EQUAL(parsed.isPair(), original.isPair());

		BOOST_REQUIRE_EQUAL(parsed, original);
		parsed.setGenerator("trash");
		BOOST_REQUIRE(!(parsed == original));
		parsed.setGenerator(original.getGenerator());
		parsed.setDistance(999.9,MILLIMETER);
		BOOST_REQUIRE(!(parsed == original));

	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("Legs.xml"), Legs::getBaseTag());
	Legs parsed = extract<Legs>(fileContents);

	string name = Legs::getPerceptName();

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getAngle(RADIAN), tolerance);
	BOOST_REQUIRE_CLOSE((float) 77.23, parsed.getDistance(MILLIMETER), tolerance);
	BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getAngleVariance(RADIAN), tolerance);
	BOOST_REQUIRE_CLOSE((float) 1.34, parsed.getDistanceVariance(MILLIMETER), tolerance);
	BOOST_REQUIRE_EQUAL(2134, parsed.getNumInvovled());
	BOOST_REQUIRE_EQUAL(4543, parsed.getInvolved1());
	BOOST_REQUIRE_EQUAL(1223, parsed.getInvolved2());
	BOOST_REQUIRE(parsed.isPair());

}

BOOST_AUTO_TEST_SUITE_END()
